SystemOrganization addCategory: #'Seaside-Session'!

WAObject subclass: #WACurrentSession
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WACurrentSession commentStamp: 'jf 9/11/2008 14:58' prior: 0!
Available for backward compatibility. You should now use "self session" in any subclass of WAObject or "WACurrentRequestContext value session" anywhere else.!

!WACurrentSession class methodsFor: 'evaluating' stamp: 'jf 9/11/2008 20:21'!
value
	| request |
	self deprecatedApi: 'Use "self session" on a subclass of WAObject or "WACurrentRequestContext value session"'.
	request := WACurrentRequestContext value.
	^ request isNil ifFalse: [ request session ]! !

WAObject subclass: #WAMain
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WAMain commentStamp: 'rew 10/19/2007 10:49' prior: 0!
WAMain subclasses are used to initialize applications when a new session is started.

Subclasses must implement the following messages:
	start:
		Does the initializing 
!

!WAMain methodsFor: 'processing' stamp: 'lr 10/28/2007 14:42'!
start: aRequest
	"The main entry point into a session."

	self subclassResponsibility! !

!WAObject methodsFor: '*Seaside-Session' stamp: 'pmm 9/1/2008 13:27'!
application
	"Answer the current seaside application, instance of WAApplication or a subclass."
	
	^self requestContext application! !

!WAObject methodsFor: '*Seaside-Session' stamp: 'pmm 9/1/2008 13:26'!
session
	"Answer the current seaside session, instance of WASesssion or a subclass."
	
	^self requestContext session! !

!WARequestContext methodsFor: '*Seaside-Session' stamp: 'jf 7/16/2008 10:49'!
application
	"Answer the current application or nil."

	^ handlers 
		detect: [ :each | each isApplication ]
		ifNone: [ nil ]! !

!WARequestContext methodsFor: '*Seaside-Session' stamp: 'jf 7/25/2008 20:34'!
session
	"Answer the current session or nil."

	^ handlers 
		detect: [ :each | each isSession ]
		ifNone: [ nil ]! !

WASystemConfiguration subclass: #WAApplicationConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WAApplicationConfiguration commentStamp: 'jf 7/22/2008 12:40' prior: 0!
WAApplicationConfiguration defines attributes required by WAApplication. All applications should include this configuration (this is done by default for new applications).

See #addAttributes: for details on provided attributes.!

!WAApplicationConfiguration methodsFor: 'description' stamp: 'pmm 9/1/2008 22:48'!
describeOn: config
	(config string: #charSet)
		label: 'Charater Set';
		comment: 'The character set reported to the browser. You may additionally need to configure your server that this is also the character set used by your server.';
		default: WAResponse defaultValueForCharSet.
	(config collection: #libraries)
		options: [ self libraryClasses ];
		default: (Set with: WAStandardFiles).
	(config string: #mimeType)
		label: 'MIME Type';
		comment: 'the Internet media type of HTML documents reported to the browser';
		default: WAResponse defaultValueForMimeType.
	(config string: #resourceBaseUrl)
		group: #server;
		comment: 'Sets the base URL for URLS generated with #resourceUrl:'.
	(config list: #sessionClass)
		comment: 'Sessions in this application will be instances of this class.';
		options: [ self sessionClasses ];
		default: WASession.
! !

!WAApplicationConfiguration methodsFor: 'description' stamp: 'jf 7/22/2008 12:00'!
libraryClasses
	^ WALibrary allSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

!WAApplicationConfiguration methodsFor: 'ancestry' stamp: 'jf 7/29/2008 12:28'!
parents
	^ Array with: WARequestHandlingConfiguration instance! !

!WAApplicationConfiguration methodsFor: 'description' stamp: 'jf 7/22/2008 12:00'!
sessionClasses
	^ WASession withAllSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

WASystemConfiguration subclass: #WASessionConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WASessionConfiguration commentStamp: 'jf 7/22/2008 12:30' prior: 0!
WASessionConfiguration defines attributes required by WASession and its subclasses. Any application using a WASession should include this configuration (this is done by default for new applications).

See #addAttributes: for details on provided attributes.!

!WASessionConfiguration methodsFor: 'description' stamp: 'jf 9/17/2008 20:09'!
describeOn: config
	(config list: #exceptionHandler)
		options: [self exceptionHandlerClasses];
		default: WAErrorHandler.
	(config list: #mainClass)
		options: [self mainClasses].
	(config number: #sessionContinuationCacheSize)
		comment: 'The number of continuations to keep cached for each session. Once a continuation has expired, users of the application will be unable to click links, submit forms, or trigger actions on that page.';
		default: 20.
	(config number: #sessionExpirySeconds)
		comment: 'The number of seconds after which inactive sessions are expired.';
		default: 600.
	(config boolean: #useSessionCookie)
		comment: 'Whether the session identifier should be stored in a cookie instead of the URL. Will fall back to URL if the client does not support cookies.';
		default: false.
	(config boolean: #sessionAllowTermination)
		comment: 'Whether sessions allows a special HTTP request to terminate the process currently running in the session. This is used during development and should be disabled for deployment.';
		default: false.! !

!WASessionConfiguration methodsFor: 'description' stamp: 'jf 9/17/2008 20:10'!
exceptionHandlerClasses
	^ WAExceptionHandler withAllSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

!WASessionConfiguration methodsFor: 'description' stamp: 'lr 11/18/2007 15:18'!
mainClasses
	^ WAMain allSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

!WASessionConfiguration methodsFor: 'ancestry' stamp: 'jf 8/5/2008 08:49'!
parents
	^ Array with: WARequestHandlingConfiguration instance! !

WARegistry subclass: #WAApplication
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WAApplication commentStamp: 'rew 10/19/2007 10:47' prior: 0!
WAApplication is the starting point for a Seaside application. When a WAComponent is registered as a top level component a WAApplication object is added to a WADispatcher. The dispatcher forwards all requests to the WAApplication, which in turn forwards them to the correct WASession object. WAApplication's parent class WARegistry maintains a list of all active sessions to the application. 

"configuration" contains a chain of WAConfituration classes that define attributes of the application. The attribute "rootComponent", for example, defines the top level WAComponent class for the application. The configuration chain includes WAUserConfiguration, WAGlobalConfiguration, WARenderLoopConfiguration and WASessionConfiguration. Other configurations can be added to the chain when the top level application is registered with a dispatcher. (See below)

"libraries" is a collection of WALibrary classes, which are used to serve css, javascript and images used by the application. These may be in methods or in files. Sometimes these libraries are replaced by static files served by Apache. See WAFileLibrary class comment for more information.

Registering an Application.
	An application can be registered with a dispatcher by using the Seaside configuration page or via code. Below MyComponent is a subclass of WAComponent. The following registers the component as an application, gives some values to attributes (or preferences) and adds a library and a configuration. 

MyComponent class>>initialize
	"self initialize"
	| application |
	application := self registerAsApplication: 'sample'.
	application preferenceAt: #sessionClass put: Glorp.WAGlorpSession.
	application addLibrary: SampleLibrary.
	application configuration addAncestor: GlorpConfiguration new.
	application preferenceAt: #glorpDatabasePlatform put: Glorp.PostgreSQLPlatform.
	application preferenceAt: #databaseServer put: '127.0.0.1'.
	application preferenceAt: #databaseConnectString put: 'glorptests'.

MyComponent>>someInstanceMethod
	"example of how to access attributes (preferences)"
	self session application preferenceAt: #glorpDatabasePlatform!

!WAApplication class methodsFor: 'accessing' stamp: 'lr 11/16/2006 18:38'!
description
	^ 'Application'! !

!WAApplication methodsFor: 'libraries' stamp: 'lr 11/18/2007 15:24'!
addLibrary: aLibraryClass
	self preferenceAt: #libraries put: (self libraries copyWith: aLibraryClass)! !

!WAApplication methodsFor: 'configuration' stamp: 'pmm 4/6/2008 15:11'!
charSet
	^ self preferenceAt: #charSet! !

!WAApplication methodsFor: 'configuration' stamp: 'jf 9/22/2008 12:54'!
contentType
	^ self mimeType seasideMimeType
		charset: self charSet;
		yourself! !

!WAApplication methodsFor: 'configuration' stamp: 'MAD 9/13/2008 15:02'!
defaultConfiguration
	^ super defaultConfiguration
		removeParent: WARequestHandlingConfiguration instance;
		addParent: WAApplicationConfiguration instance;
		addParent: WASessionConfiguration instance;
		yourself! !

!WAApplication methodsFor: 'accessing' stamp: 'lr 9/14/2008 13:25'!
description
	^ [
		| rootClass |
		rootClass := self preferenceAt: #rootClass.
		rootClass isNil
			ifTrue: [ String new ]
			ifFalse: [ rootClass description ] ]
		on: WAAttributeNotFound
		do: [ :err | String new ]! !

!WAApplication methodsFor: 'request handling' stamp: 'lr 9/13/2008 11:27'!
handleDefaultRequest: aRequestContext
	^ (self sessionClass new setParent: self) handleRequest: aRequestContext! !

!WAApplication methodsFor: 'as yet unclassified' stamp: 'jf 9/13/2008 14:30'!
handleExpiredRequest: aRequestContext
	| response sessionCookie |
	response := super handleExpiredRequest: aRequestContext.
	sessionCookie := self sessionCookieFor: aRequestContext request.
	sessionCookie isNil 
		ifFalse: [ response deleteCookie: sessionCookie ].
	^ response! !

!WAApplication methodsFor: 'accessing' stamp: 'pmm 3/22/2008 15:13'!
handlerCookieName
	"returns the key of the session cookie"
	^ self handlerField! !

!WAApplication methodsFor: 'testing' stamp: 'mb 4/5/2006 20:26'!
isApplication
	^true! !

!WAApplication methodsFor: 'configuration' stamp: 'lr 2/10/2008 15:59'!
libraries
	^ self preferenceAt: #libraries! !

!WAApplication methodsFor: 'configuration' stamp: 'pmm 4/6/2008 15:11'!
mimeType
	^ self preferenceAt: #mimeType! !

!WAApplication methodsFor: 'libraries' stamp: 'lr 11/18/2007 15:25'!
removeLibrary: aLibraryClass
	self preferenceAt: #libraries put: (self libraries copyWithout: aLibraryClass)! !

!WAApplication methodsFor: 'configuration' stamp: 'lr 2/12/2007 22:02'!
resourceBaseUrl
	^ self preferenceAt: #resourceBaseUrl! !

!WAApplication methodsFor: 'request handling' stamp: 'lr 9/13/2008 11:13'!
responseForContext: aRequestContext 
	| cookie |
	cookie := self sessionCookieFor: aRequestContext request.
	cookie isNil ifFalse: [
		aRequestContext request fields 
			at: self handlerField
			ifAbsentPut: [ cookie value ] ].
	^ super responseForContext: aRequestContext! !

!WAApplication methodsFor: 'configuration' stamp: 'lr 10/28/2007 14:42'!
sessionClass
	^ self preferenceAt: #sessionClass! !

!WAApplication methodsFor: 'private' stamp: 'jf 9/22/2008 21:08'!
sessionCookieFor: aRequest
	"answers the session cookie or nil
	There can be several cookies with the correct key (_s). We try to find the one who has the most specific path."

	| candidates requestUrl |
	candidates := aRequest cookiesAt: self handlerCookieName.
	
	candidates isEmptyOrNil
		ifTrue: [ ^nil ].
	candidates size = 1 
		ifTrue: [ ^candidates first ].
		
	requestUrl := aRequest url.
	candidates := candidates collect: [ :each |
		each -> (each path seasideUrl relativeTo: requestUrl) ].
	
	candidates := candidates reject: [ :each |
		| path |
		path := each value.
		path notEmpty and: [ path first = '..' ] ].

	candidates isEmpty
		ifTrue: [  ^nil ].
		
	candidates size > 1
		ifTrue: [
			candidates sort: [ :a :b |
				 a value size <= b value size ] ].
	
	^candidates first key! !

!WAApplication methodsFor: 'request handling' stamp: 'lr 4/9/2007 10:51'!
updateRoot: anHtmlRoot
	self libraries do: [ :each | each default updateRoot: anHtmlRoot ]! !

!WAApplication methodsFor: 'request handling' stamp: 'lr 4/15/2007 16:21'!
updateUrl: anUrl! !

WAExpiringHandler subclass: #WASession
	instanceVariableNames: 'continuations monitor jumpTo cookiesEnabled'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WASession commentStamp: 'jf 7/22/2008 11:48' prior: 0!
I am a Seaside session. A new instance of me gets created when an user accesses an application for the first time and is persistent as long as the user is interacting with it.

This class is intended to be subclasses by applications that need global state, like a user. Custom state can be added by creating instance variables and storing it there. The session can be retrieved by #session if inside a component or task or by evaluating: WACurrentRequestContext session

If the session has not been used for #defaultTimeoutSeconds, it is garbage collected by the system. To manually expire a session call #expire.

A good way to clear all sessions is the following code:

WARegistry clearAllHandlers.
Smalltalk garbageCollect!

!WASession methodsFor: 'accessing' stamp: 'lr 4/15/2007 15:55'!
actionField
	^ '_k'! !

!WASession methodsFor: 'accessing' stamp: 'lr 8/31/2008 18:19'!
actionUrlForContinuation: aContinuation
	^ self actionUrlForKey: (continuations store: aContinuation)! !

!WASession methodsFor: 'accessing' stamp: 'lr 4/9/2008 10:48'!
actionUrlForKey: aString
	| url |
	url := self application
		urlForRequestHandler: self
		addHandlerField: self useSessionCookie not.
	url takeServerParametersFromRequest: self requestContext request.
	url addParameter: self actionField value: aString.
	jumpTo ifNotNil: [ url fragment: jumpTo ].
	^ url! !

!WASession methodsFor: 'deprecated' stamp: 'lr 4/9/2008 10:48'!
addLoadScript: aString
	self deprecatedApi: 'Use WAContext>>#addLoadScript:'.
	^ self requestContext addLoadScript: aString! !

!WASession methodsFor: 'accessing' stamp: 'lr 4/15/2007 15:55'!
application
	^ parent! !

!WASession methodsFor: 'accessing' stamp: 'lr 4/15/2007 15:55'!
baseUrl
	^ self parent baseUrl! !

!WASession methodsFor: 'private' stamp: 'lr 7/10/2007 19:34'!
checkCookiesField
	^ '_c'! !

!WASession methodsFor: 'scripts' stamp: 'lr 4/15/2007 15:55'!
clearJumpTo
	jumpTo := nil! !

!WASession methodsFor: 'responding' stamp: 'pmm 5/23/2007 23:50'!
closePopup
	^ self respond: [ :url |
		WAResponse new
			nextPutAll: '<html><script>';
			nextPutAll: 'self.close();';
			nextPutAll: 'self.opener.location=self.opener.location';
			nextPutAll: '</script></html>';
			yourself]! !

!WASession methodsFor: 'responding' stamp: 'pmm 8/31/2008 14:48'!
closePopupAndContinue
	^ self respond: [ :url | 
		WAResponse new
			nextPutAll: '<html><script>';
			nextPutAll: 'self.close();';
			nextPutAll: 'self.opener.location=';
			nextPutAll: url seasideString printString;
			nextPutAll: '</script></html>';
			yourself ]! !

!WASession methodsFor: 'responding' stamp: 'pmm 5/23/2007 23:50'!
closePopupWithoutReloadingOpener
	^ self respond: [ :url |
		WAResponse new
			nextPutAll: '<html><script>';
			nextPutAll: 'self.close();';
			nextPutAll: '</script></html>';
			yourself]! !

!WASession methodsFor: 'deprecated' stamp: 'pmm 7/23/2008 06:46'!
currentRequest
	"returns the current WARequest"
	self deprecatedApi: 'use WARequestContext >> #request directly'.
	^self requestContext request! !

!WASession methodsFor: 'expiring' stamp: 'jf 7/28/2008 17:11'!
defaultTimeoutSeconds
	^ self sessionExpirySeconds isNil
		ifFalse: [ self sessionExpirySeconds ]
		ifTrue: [ super defaultTimeoutSeconds ]! !

!WASession methodsFor: 'configuration' stamp: 'jf 9/17/2008 20:12'!
exceptionHandler
	^ self preferenceAt: #exceptionHandler! !

!WASession methodsFor: 'request handling' stamp: 'jf 9/17/2008 20:13'!
incomingRequest: aRequestContext 
	((aRequestContext request fields includesKey: 'terminate') and: [ self sessionAllowTermination ]) ifTrue: [
		monitor terminate.
		^ WAResponse new nextPutAll: 'Process terminated' ].
	^ monitor 
		critical:  [
			WACurrentRequestContext 
				use: aRequestContext
				during: [ 
					self withEscapeHandler: [
						self exceptionHandler handleExceptionsDuring: [
							self performRequest: aRequestContext request ] ] ] ]
		ifError:  [ :error | 
			WACurrentRequestContext 
				use: aRequestContext
				during: [ self exceptionHandler internalError: error ] ]! !

!WASession methodsFor: 'initializing' stamp: 'lr 7/10/2007 19:28'!
initialize
	super initialize.
	continuations := WALRUCache new.
	monitor := WAProcessMonitor new.
	cookiesEnabled := true! !

!WASession methodsFor: 'testing' stamp: 'jf 7/25/2008 20:33'!
isSession
	^ true! !

!WASession methodsFor: 'scripts' stamp: 'lr 4/15/2007 15:55'!
jumpToAnchor: aString
	jumpTo := aString! !

!WASession methodsFor: 'configuration' stamp: 'jf 7/28/2008 17:07'!
mainClass
	^ self preferenceAt: #mainClass! !

!WASession methodsFor: 'responding' stamp: 'lr 4/9/2008 10:48'!
newSessionUrl
	^ self baseUrl takeServerParametersFromRequest: self requestContext request! !

!WASession methodsFor: 'responding' stamp: 'jf 7/25/2008 20:02'!
pageIntentionallyLeftBlank
	self requestContext returnResponse: WAResponse new! !

!WASession methodsFor: 'processing' stamp: 'jf 9/11/2008 13:15'!
performRequest: aRequest
	| key continuation |
	key := aRequest fields
		at: self actionField
		ifAbsent: [ ^ self start: aRequest ].
	((aRequest fields includesKey: self checkCookiesField)
		and: [ aRequest cookies isEmpty ])
			ifTrue: [ cookiesEnabled := false ].
	(key isString not and: [ key isSequenceable ] )
		ifTrue: [ key := key first ].
	continuation := continuations
		at: key
		ifAbsent: [ ^ self unknownRequest: aRequest ].
	^ continuation value: aRequest! !

!WASession methodsFor: 'configuration' stamp: 'jf 7/28/2008 17:06'!
preferenceAt: aSymbol
	^ self application isNil ifFalse: [ self application preferenceAt: aSymbol ]! !

!WASession methodsFor: 'redirecting' stamp: 'lr 5/10/2007 22:20'!
redirect
	^ self respond: [ :url | self redirectResponseFor: url ]! !

!WASession methodsFor: 'redirecting' stamp: 'jf 9/22/2008 21:05'!
redirectResponseFor: aUrlOrString
	^ self application redirectResponseFor: self requestContext to: aUrlOrString! !

!WASession methodsFor: 'redirecting' stamp: 'jf 9/22/2008 21:04'!
redirectTo: aUrlOrString
	self requestContext returnResponse: (self redirectResponseFor: aUrlOrString)! !

!WASession methodsFor: 'redirecting' stamp: 'lr 7/10/2007 19:38'!
redirectWithCookie: aCookie
	| response |
	self respond: [ :url |
		url addParameter: self checkCookiesField.
		response := self redirectResponseFor: url.
		response addCookie: aCookie.
		response ]! !

!WASession methodsFor: 'redirecting' stamp: 'jf 9/14/2008 00:06'!
redirectWithMessage: aString delay: aNumber 
	self respond: [ :url | 
		self application responseFactory
			refreshAfter: aNumber
			location: url
			message: aString ]! !

!WASession methodsFor: 'responding' stamp: 'lr 3/29/2008 14:20'!
respond: aBlock 
	^ WAContinuation currentDo: [ :cc | self respond: aBlock onAnswer: cc ]! !

!WASession methodsFor: 'responding' stamp: 'jf 7/25/2008 20:04'!
respond: aBlock onAnswer: aContinuation
	| url response |
	url := self actionUrlForContinuation: aContinuation.
	response := aBlock value: url.
	self requestContext returnResponse: response! !

!WASession methodsFor: 'deprecated' stamp: 'jf 7/14/2008 17:06'!
returnResponse: aResponse
	self deprecatedApi: 'Use WARequestContext>>returnResponse: aResponse'.
	self requestContext returnResponse: aResponse! !

!WASession methodsFor: 'configuration' stamp: 'jf 8/5/2008 08:46'!
sessionAllowTermination
	^ self preferenceAt: #sessionAllowTermination! !

!WASession methodsFor: 'configuration' stamp: 'jf 7/28/2008 17:08'!
sessionContinuationCacheSize
	^ self preferenceAt: #sessionContinuationCacheSize! !

!WASession methodsFor: 'accessing' stamp: 'lr 8/30/2008 16:22'!
sessionCookie
	"creates a new session cookie"
	^ self requestContext newCookie
		key: self application handlerCookieName;
		value: (self application ensureKeyForHandler: self) seasideString;
		discard: true;
		comment: 'the Seaside Session Cookie';
		commentUrl: 'http://www.seaside.st/documentation/sessionCookie';
		yourself! !

!WASession methodsFor: 'configuration' stamp: 'jf 7/28/2008 17:10'!
sessionExpirySeconds
	^ self preferenceAt: #sessionExpirySeconds! !

!WASession methodsFor: 'initializing' stamp: 'jf 7/29/2008 14:58'!
setParent: aRequestHandler
	super setParent: aRequestHandler.
	continuations capacity: self sessionContinuationCacheSize.! !

!WASession methodsFor: 'processing' stamp: 'lr 10/28/2007 14:42'!
start: aRequest
	self useSessionCookie
		ifTrue: [ self redirectWithCookie: self sessionCookie ].
	^ self mainClass new start: aRequest! !

!WASession methodsFor: 'request handling' stamp: 'lr 8/31/2008 17:30'!
unknownRequest: aRequest
	^ self start: aRequest! !

!WASession methodsFor: 'private' stamp: 'lr 4/15/2007 15:55'!
unregistered
	super unregistered.
	self expire! !

!WASession methodsFor: 'updating' stamp: 'jf 9/22/2008 13:02'!
updateRoot: anHtmlRoot
	self application updateRoot: anHtmlRoot.
	anHtmlRoot meta contentType: self application contentType.
	anHtmlRoot meta contentScriptType: WAMimeType textJavascript.
	anHtmlRoot htmlAttributes
		at: 'xmlns' put: 'http://www.w3.org/1999/xhtml';
		at: 'xml:lang' put: 'en';
		at: 'lang' put: 'en'.
	anHtmlRoot beXhtml10Strict.
	anHtmlRoot title: 'Seaside'! !

!WASession methodsFor: 'updating' stamp: 'lr 8/31/2008 14:49'!
updateStates: aSnapshot
	| current |
	current := self filter.
	[ current = self ] whileFalse: [
		current updateStates: aSnapshot.
		current := current next ].
	aSnapshot register: filter! !

!WASession methodsFor: 'updating' stamp: 'lr 4/15/2007 16:27'!
updateUrl: anUrl
	self application updateUrl: anUrl! !

!WASession methodsFor: 'configuration' stamp: 'jf 7/28/2008 17:09'!
useSessionCookie
	^ cookiesEnabled and: [ self preferenceAt: #useSessionCookie ]! !

!WASession methodsFor: 'deprecated' stamp: 'jf 9/17/2008 20:21'!
withErrorHandler: aBlock
	self deprecatedApi: 'ExceptionHandlers should be allowed to set up their own exception handling code. See WAExceptionHandler class>>handleExceptionsDuring:'.
	^ self exceptionHandler handleExceptionsDuring: aBlock! !
